# -*- coding: utf-8 -*-
#  Copyright (C) 2007, 2008 Stewart Adam
#  This file is part of fwfstab.

#  fwfstab is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.

#  fwfstab is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#  GNU General Public License for more details.

#  You should have received a copy of the GNU General Public License
#  along with fwfstab; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""
Custom GUI widgets or widgets handling and wrappers
"""
import datetime
import gtk
import gobject
import os
import pango
import sys
import traceback
#--
import fwfstab
from fwfstab.i18n import _
from fwfstab import interface

def setupEntriesTreeview(treeview):
  """ Sets up the treview """
  liststore = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING,
                            gobject.TYPE_STRING, gobject.TYPE_STRING,
                            gobject.TYPE_STRING, gobject.TYPE_STRING)
  cell = gtk.CellRendererText()
  col = gtk.TreeViewColumn("Device", cell, text=0)
  col.set_sort_column_id(0)
  col.set_resizable(True)
  treeview.append_column(col)
  cell = gtk.CellRendererText()
  col = gtk.TreeViewColumn("Mount Point", cell, text=1)
  col.set_sort_column_id(1)
  col.set_resizable(True)
  treeview.append_column(col)
  cell = gtk.CellRendererText()
  col = gtk.TreeViewColumn("Filesystem", cell, text=2)
  col.set_sort_column_id(2)
  col.set_resizable(True)
  treeview.append_column(col)
  cell = gtk.CellRendererText()
  col = gtk.TreeViewColumn("Mount Options", cell, text=3)
  cell.set_property('ellipsize', pango.ELLIPSIZE_END)
  col.set_sort_column_id(3)
  col.set_resizable(True)
  # makes it take as much space as possible
  col.set_expand(True)
  treeview.append_column(col)
  cell = gtk.CellRendererText()
  col = gtk.TreeViewColumn("Freq", cell, text=4)
  col.set_sort_column_id(4)
  col.set_resizable(False)
  col.set_max_width(100)
  treeview.append_column(col)
  cell= gtk.CellRendererText()
  col = gtk.TreeViewColumn("Pass No.", cell, text=5)
  col.set_sort_column_id(5)
  col.set_resizable(False)
  col.set_max_width(100)
  treeview.append_column(col)
  treeview.set_model(liststore)

def setupOptionsTreeview(treeview):
  """ Sets up the treview """
  liststore = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING)
  ## Options treeview
  cell = gtk.CellRendererText()
  col = gtk.TreeViewColumn("Option Name", cell, text=0)
  col.set_sort_column_id(0)
  col.set_resizable(True)
  treeview.append_column(col)
  cell = gtk.CellRendererText()
  col = gtk.TreeViewColumn("Description", cell, text=1)
  col.set_sort_column_id(1)
  col.set_resizable(True)
  treeview.append_column(col)
  treeview.set_model(liststore)

# *-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*

class StatusBar:
  """ Status bar wrapper """
  def __init__(self, widget):
    """ Initialize.
      widget: GTK widget to use as the statusbar
    """
    self.statusbar = widget

  def newmessage(self, message, seconds=3):
    """ New message in status bar.
      message: Message to display
      seconds: number of seconds to display the message
           defaults to three.
    """
    try:
      self.message_timeout()
    # Just means there was no existing message.
    except AttributeError:
      pass
    self.statusbar.push(1, message)
    self.message_timer = gobject.timeout_add(seconds*1000, self.message_timeout)

  def message_timeout(self):
    """ Remove a message from the statusbar """
    self.statusbar.pop(1)
    gobject.source_remove(self.message_timer)
    return True

# *-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*

class ProgressBar:
  """ Progress bar wrapper """
  def __init__(self, widget, ms=15):
    """ Initialize.
      widget: GTK widget to use as the progressbar
      ms: Pulse step is every this many miliseconds
    """
    self.progressbar = widget
    self.ms = ms
    self.pulsing = False
    self.progressbar.set_pulse_step(0.01)

  def set_pulse_step(self, step):
    """ Wrapper: See GTK+ help """
    return self.progressbar.set_pulse_step(step)

  def set_fraction(self, fraction):
    """ Wrapper: See GTK+ help """
    return self.progressbar.set_fraction(fraction)

  def _pulse(self):
    """ Pulse the progressbar. """
    self.progressbar.pulse()
    # Has to return true to happen again
    return True

  def startPulse(self):
    """ Start auto-pulsing the progressbar """
    if self.pulsing:
      self.stopPulse()
    self.pulsing = True
    self.pulsetimer = gobject.timeout_add(self.ms, self._pulse)

  def stopPulse(self):
    """ Stop auto-pulsing the progressbar """
    gobject.source_remove(self.pulsetimer)
    self.progressbar.set_fraction(0)
    self.pulsing = False
    return True

  def setMs(self, ms):
    """ Set the ms value (see __init__) """ 
    self.ms = int(ms)
    if self.pulsing:
      self.stopPulse()
      self.startPulse()

  def set_text(self, text):
    """ Set the progressbar's text """
    self.progressbar.set_text(text)

# *-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*

class GenericDia:
  """ Wrapper for the generic dialog """
  def __init__(self, dialog, title, parent):
    """ Initialize """
    self.dialog = dialog
    self.dialog.set_title(title)
    self.dialog.set_transient_for(parent)

  def run(self):
    """ Run the dialog """
    self.dialog.show()
    try:
      return self.dialog.run()
    except:
      self.destroy()
      raise

  def destroy(self):
    """ Hide the dialog, don't kill it! """
    self.dialog.hide()

  def runAndDestroy(self):
    """ Executes run() and destroy() """
    response = self.run()
    self.destroy()
    return response

# *-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*

def displayError(self, header, message):
  """ Display the error window and set the labels """
  self.ui.errorHeaderLabel.set_text('<span size="x-large" weight="bold">%s</span>' % header)
  self.ui.errorHeaderLabel.set_use_markup(True)
  self.ui.errorMessageLabel.set_text(message)
  self.ui.errorMessageLabel.set_use_markup(True)
  dialog = fwfstab.Dialog(self.ui.error)
  dialog.run()
  dialog.destroy()

def displayConfirm(self, header, message):
  """ Display the confirm window and set the labels """
  self.ui.confirmHeaderLabel.set_text('<span size="x-large" weight="bold">%s</span>' % header)
  self.ui.confirmHeaderLabel.set_use_markup(True)
  self.ui.confirmMessageLabel.set_text(message)
  self.ui.confirmMessageLabel.set_use_markup(True)
  dialog = fwfstab.Dialog(self.ui.confirm)
  retval = dialog.run()
  dialog.destroy()
  return retval

# *-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*

class bugReport(GenericDia):
  """ Wrapper for a bug report dialog """
  def __init__(self, dialog, textview, parent, tracebackText):
    """ Uses `dialog' to show bug report `traceback' in `textview' on top
        of `parent' """
    GenericDia.__init__(self, dialog, _('Bug Report - fwfstab'), parent)
    textview.get_buffer().set_text(tracebackText)

# *-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*

def reportBug():
  """ Report a bug dialog """
  c = interface.Controller('%s/BugReport.glade' % os.getcwd(), 'bugreport')
  (type, value, tb) = sys.exc_info()
  tracebackText = ''.join(traceback.format_exception(type, value, tb))
  reportWindow = bugReport(c.ui.bugreport, c.ui.bugreportTextview, None, tracebackText)
  response = reportWindow.runAndDestroy()
  if response == gtk.RESPONSE_OK:
    filename = saveFilename(c.ui.bugreport)
    if not filename:
      return
    if fwfstab.CheckPerms(filename):
      fh = open(filename, 'w')
      fh.write(_(datetime.datetime.today().strftime('fwfstab bug report written saved at %I:%M %p on %Y-%m-%d\n\n')))
      fh.write(tracebackText)
      fh.close()
    else:
      displayError(self, _('Could not write bug report'), _('You do not have sufficent permissions to write to the file.\nThe bug report will not be saved.'))
  sys.exit(1)
